<?php
/**
 * 2007-2018 PrestaShop
 *
 * NOTICE OF LICENSE
 *
 * This source file is subject to the Academic Free License (AFL 3.0)
 * that is bundled with this package in the file LICENSE.txt.
 * It is also available through the world-wide-web at this URL:
 * http://opensource.org/licenses/afl-3.0.php
 * If you did not receive a copy of the license and are unable to
 * obtain it through the world-wide-web, please send an email
 * to license@prestashop.com so we can send you a copy immediately.
 *
 * DISCLAIMER
 *
 * Do not edit or add to this file if you wish to upgrade PrestaShop to newer
 * versions in the future. If you wish to customize PrestaShop for your
 * needs please refer to http://www.prestashop.com for more information.
 *
 * @author    PrestaShop SA <contact@prestashop.com>
 * @copyright 2007-2018 PrestaShop SA
 * @license   http://opensource.org/licenses/afl-3.0.php  Academic Free License (AFL 3.0)
 *  International Registered Trademark & Property of PrestaShop SA
 */

if (!defined('_PS_VERSION_')) {
    exit;
}

include_once(_PS_MODULE_DIR_ . '/besterongateway/lib/BesteronClass.php');

class BesteronGateway extends PaymentModule
{

    /**
     * config value Besteron CID
     *
     * @var string
     */
    private $cid;

    /**
     * config value Besteron KEY
     *
     * @var string
     */
    private $key;

    /**
     * config value demo mode
     *
     * @var boolean
     */
    private $demo = true;

    public function __construct()
    {
        $this->module_key = 'd392c810e2df5beef20ca6d4f1468541';
        $this->name = 'besterongateway';
        $this->tab = 'payments_gateways';
        $this->version = '1.0.9';
        $this->author = 'Besteron';
        $this->need_instance = 0;
        $this->ps_versions_compliancy = array('min' => '1.6', 'max' => _PS_VERSION_);
        $this->bootstrap = true;
        $this->isDemo();

        // config values
        $config = Configuration::getMultiple(array('BESTERON_KEY', 'BESTERON_CID', 'BESTERON_DEMO'));

        if (!empty($config['BESTERON_KEY']) && !empty($config['BESTERON_CID'])) {
            $this->key = $config['BESTERON_KEY'];
            $this->cid = $config['BESTERON_CID'];
            $this->demo = $config['BESTERON_DEMO'];
        }

        parent::__construct();

        $this->displayName = $this->l('Besteron');
        $this->description = $this->l('Online platby Besteron');
        $this->confirmUninstall = $this->l('Are you sure you want to uninstall?');

        if (!Configuration::get('BESTERON_KEY') || !Configuration::get('BESTERON_CID')) {
            $this->warning = $this->l('Options are not set.');
        }
    }

    public function install()
    {
        if (Shop::isFeatureActive()) {
            Shop::setContext(Shop::CONTEXT_ALL);
        }
        if (!parent::install() ||
            !$this->registerHook('payment') ||
            !$this->registerHook('displayPaymentReturn') ||
            !$this->registerHook('header') ||
            !Configuration::updateValue('BESTERON_DEMO', 1)
        ) {
            return false;
        }
        return true;
    }

    public function uninstall()
    {
        if (!$this->unregisterHook('header')
            || !$this->unregisterHook('payment')
            || !$this->unregisterHook('displayPaymentReturn')
            || !parent::uninstall()
        ) {
            return false;
        }
        return true;
    }

    /**
     * shows the configuration page in the back-end
     *
     * @return string
     */
    public function getContent()
    {
        $output = '';
        if (Tools::isSubmit('submit' . $this->name)) {
            $key = trim((string)Tools::getValue('BESTERON_KEY'));
            $cid = trim((string)Tools::getValue('BESTERON_CID'));
            if (!$cid || empty($cid)) {
                $output .= $this->displayError($this->l('Please enter the CID.'));
            } elseif (!$key || empty($key)) {
                $output .= $this->displayError($this->l('Please enter the KEY.'));
            } else {
                $demo = Tools::getValue('BESTERON_DEMO');
                Configuration::updateValue('BESTERON_KEY', $key);
                Configuration::updateValue('BESTERON_CID', $cid);
                Configuration::updateValue('BESTERON_DEMO', $demo);
                $this->key = $key;
                $this->cid = $cid;
                $this->demo = $demo;
                $output .= $this->displayConfirmation($this->l('Settings updated'));
            }
        }
        return $output . $this->displayForm();
    }

    public function displayForm()
    {
        // Get default language
        $default_lang = (int)Configuration::get('PS_LANG_DEFAULT');
        // Init Fields form array
        $fields_form = array(
            0 => array(
                'form' => array(
                    'legend' => array(
                        'title' => $this->l('Besteron Settings'),
                    ),
                    'input' => array(
                        array(
                            'type' => 'text',
                            'label' => $this->l('Besteron Client ID'),
                            'desc' => $this->l('Please enter your Besteron Client ID.'),
                            'name' => 'BESTERON_CID',
                            'size' => 10,
                            'required' => true
                        ),
                        array(
                            'type' => 'text',
                            'label' => $this->l('Besteron Key'),
                            'desc' => $this->l('Please enter your Besteron Key.'),
                            'name' => 'BESTERON_KEY',
                            'size' => 64,
                            'required' => true
                        ),
                        array(
                            'type' => 'switch',
                            'label' => $this->l('Demo Mode'),
                            'name' => 'BESTERON_DEMO',
                            'is_bool' => true,
                            'values' => array(
                                array(
                                    'id' => 'active_on',
                                    'value' => 1,
                                    'label' => $this->l('Enabled')
                                ),
                                array(
                                    'id' => 'active_off',
                                    'value' => 0,
                                    'label' => $this->l('Disabled')
                                )
                            )
                        )
                    ),
                    'submit' => array(
                        'title' => $this->l('Save'),
                        'class' => 'button'
                    )
                )
            )
        );
        $helper = new HelperForm();
        // Module, token and currentIndex
        $helper->module = $this;
        $helper->name_controller = $this->name;
        $helper->token = Tools::getAdminTokenLite('AdminModules');
        $helper->currentIndex = AdminController::$currentIndex . '&configure=' . $this->name;
        // Language
        $helper->default_form_language = $default_lang;
        $helper->allow_employee_form_lang = $default_lang;
        // Title and toolbar
        $helper->title = $this->displayName;
        $helper->show_toolbar = true;    // false -> remove toolbar
        $helper->toolbar_scroll = true;      // yes - > Toolbar is always visible on the top of the screen.
        $helper->submit_action = 'submit' . $this->name;
        $helper->toolbar_btn = array(
            'save' => array(
                'desc' => $this->l('Save'),
                'href' => AdminController::$currentIndex . '&configure=' . $this->name . '&save' . $this->name .
                    '&token=' . Tools::getAdminTokenLite('AdminModules'),
            ),
            'back' => array(
                'href' => AdminController::$currentIndex . '&token=' . Tools::getAdminTokenLite('AdminModules'),
                'desc' => $this->l('Back to list')
            )
        );

        // Load current value
        $helper->fields_value['BESTERON_CID'] = $this->cid;
        $helper->fields_value['BESTERON_KEY'] = $this->key;
        $helper->fields_value['BESTERON_DEMO'] = $this->demo ? 1 : 0;
        return $helper->generateForm($fields_form);
    }

    /**
     * include css file in frontend
     *
     * @param $params
     */
    public function hookHeader()
    {
        $this->context->controller->addCSS(($this->_path) . 'views/css/front.css', 'all');
    }

    /**
     * show besteron payment on frontend
     *
     * @param type $params
     * @return type
     */
    public function hookPayment($params)
    {
        if (!$this->active) {
            return;
        }
        if (!$this->checkCurrency($params['cart'])) {
            return;
        }
        $this->smarty->assign(array(
            'banks' => Besteron::$banks,
            'this_path_ssl' => Tools::getShopDomainSsl(true, true) . __PS_BASE_URI__ . 'modules/' . $this->name . '/'
        ));
        return $this->display(__FILE__, 'payment.tpl');
    }

    /**
     * confirm status after return from besteron gateway
     *
     * @param array $params
     * @return string
     */
    public function hookDisplayPaymentReturn($params)
    {
        if (!$this->active) {
            return;
        }
        //$total = $params['total_to_pay'];
        $order_id = $params['objOrder']->id;
        $status_error = Configuration::get('PS_OS_ERROR');
        $status_authorized = (string)Configuration::get('PS_OS_PAYMENT');
        $status = $status_error;
        $error = null;
        $result = Tools::getValue('RESULT', 'FAIL');
        if ($result == 'OK') {
            $besteron = new BesteronResponse(
                array(
                    Besteron::PARAM_CID => $this->cid,
                    Besteron::PARAM_KEY => $this->key,
                    Besteron::PARAM_DEMO_MODE => $this->demo,
                    Besteron::PARAM_AMNT => Tools::getValue('AMNT', 0),
                    Besteron::PARAM_CURR => Tools::getValue('CURR', ''),
                    Besteron::PARAM_VS => Tools::getValue('VS', ''),
                    Besteron::PARAM_TYPE => Tools::getValue('TYPE', ''),
                    Besteron::PARAM_RESULT => $result,
                    Besteron::PARAM_SIGN => Tools::getValue('SIGN', '')
                )
            );
            if (!$besteron->isValid()) {
                $error = 'invalid sign';
            } else {
                $status = $status_authorized;
            }
        }
        $history = new OrderHistory();
        $history->id_order = (int)$order_id;
        $history->changeIdOrderState($status, $order_id);
        $history->add();
        if ($status === $status_authorized) {
            Logger::addLog('Besteron module: order was paid with id_order ' . $order_id);
        } else {
            Logger::addLog('Besteron module: order cancceled with id_order ' . $order_id);
        }
        $this->smarty->assign('result', $result);
        $this->smarty->assign('error', $error);
        return $this->display(__FILE__, 'payment_return.tpl');
    }

    /**
     * check supported currency
     *
     * @param type $cart
     * @return boolean
     */
    public function checkCurrency($cart)
    {
        $currency_order = new Currency((int)$cart->id_currency);
        if ('EUR' == $currency_order->iso_code) {
            return true;
        }
        return false;
    }

    /* ------- GETTERS --------------- */

    public function getCid()
    {
        return $this->cid;
    }

    public function getKey()
    {
        return $this->key;
    }

    public function isDemo()
    {
        return $this->demo;
    }
}
