<?php
/**
 * 2007-2018 PrestaShop
 *
 * NOTICE OF LICENSE
 *
 * This source file is subject to the Academic Free License (AFL 3.0)
 * that is bundled with this package in the file LICENSE.txt.
 * It is also available through the world-wide-web at this URL:
 * http://opensource.org/licenses/afl-3.0.php
 * If you did not receive a copy of the license and are unable to
 * obtain it through the world-wide-web, please send an email
 * to license@prestashop.com so we can send you a copy immediately.
 *
 * DISCLAIMER
 *
 * Do not edit or add to this file if you wish to upgrade PrestaShop to newer
 * versions in the future. If you wish to customize PrestaShop for your
 * needs please refer to http://www.prestashop.com for more information.
 *
 * @author    PrestaShop SA <contact@prestashop.com>
 * @copyright 2007-2018 PrestaShop
 * @license   http://opensource.org/licenses/afl-3.0.php  Academic Free License (AFL 3.0)
 *  International Registered Trademark & Property of PrestaShop SA
 */

class BesteronGatewayValidationModuleFrontController extends ModuleFrontController
{

    public function postProcess()
    {
        //validate bic code - key
        $type = $_REQUEST['besteron_type'];
        if (!$type || !array_key_exists($type, Besteron::$banks)) {
            //return back to payment
            $error = 'Error: Please select your bank from which to make a payment';
            Tools::redirect(Tools::getShopDomain(true, false) . __PS_BASE_URI__
                . 'module/besterongateway/payment?error=' . urlencode($error));
            return;
        }
        $cart = $this->context->cart;
        $customer = new Customer($cart->id_customer);
        if (!Validate::isLoadedObject($customer)) {
            Tools::redirect('index.php?controller=order&step=1');
            return;
        }
        $amnt = $cart->getOrderTotal(true, Cart::BOTH);
        $currency = $this->context->currency;
        $this->module->validateOrder(
            $cart->id,
            Configuration::get('PS_OS_BANKWIRE'),
            $amnt,
            $this->module->displayName,
            null,
            array(),
            (int)$currency->id,
            false,
            $customer->secure_key
        );
        $ru = Tools::getShopDomain(true, false) . __PS_BASE_URI__
            . 'index.php?controller=order-confirmation&id_cart=' . $cart->id
            . '&id_module=' . $this->module->id
            . '&id_order=' . $this->module->currentOrder
            . '&key=' . $customer->secure_key;
        $vs = str_pad($this->module->currentOrder, 10, '0', STR_PAD_LEFT);

        $email = $customer->email;

        try {
            $besteron = new BesteronRequest(
                array(
                    Besteron::PARAM_CID => trim($this->module->getCid()),
                    Besteron::PARAM_KEY => trim($this->module->getKey()),
                    Besteron::PARAM_DEMO_MODE => $this->module->isDemo(),
                    Besteron::PARAM_AMNT => 0 + $amnt,
                    Besteron::PARAM_CURR => Besteron::CURR_EURO,
                    Besteron::PARAM_VS => $vs,
                    Besteron::PARAM_RU => $ru,
                    Besteron::PARAM_EMAIL => $email
                )
            );
            $url = $besteron->getUrl($type);
        } catch (Exception $e) {
            $error = $e->getMessage();
            Tools::redirect(Tools::getShopDomain(true, false) . __PS_BASE_URI__
                  . 'module/besterongateway/payment?error=' . urlencode($error));
        }
        Tools::redirect($url);
    }
}
